/***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef I_ACTIONSBOX_H
#define I_ACTIONSBOX_H

#include <Plasma/Frame>
#include <Plasma/PushButton>
#include <Plasma/Applet>

#include <QHash>

class ActionsBox: public Plasma::Frame
{
    Q_OBJECT

    enum State {
        Initialized = 0,
        WaitingToAppear = 1,
        FadingIn = 2,
        Running = 3,
        FadingOut = 4,
        Dead = 5,
        StateCount = 6
    };


    public:
        Q_ENUMS( ActionBoxPosition );

        enum ActionBoxPosition {
            TopLeft = 0,
            TopCenter = 1,
            TopRight = 2,
            BottomLeft = 3,
            BottomCenter = 4,
            BottomRight = 5,
            PositionCount = 6
        };


        static ActionsBox* Instance( ActionBoxPosition boxPos, Plasma::Applet *parent);
        void exec();
        void hideActions();
        void kill();

    protected:
        explicit ActionsBox( ActionBoxPosition boxPos, Plasma::Applet *parent = 0 );
        virtual ~ActionsBox();

    private slots:
        void openWebsite();
        //void savePhoto();
        void showNextEngine();
        void showPrevEngine();

        void appearTimeout();
        void fadeInFinished();
        void fadeOutFinished();

    private:
        State m_state;

        Plasma::PushButton *m_open;
        //Plasma::IconWidget *m_save;
        Plasma::PushButton *m_nextEngine;
        Plasma::PushButton *m_prevEngine;

        QTimer *m_appearaceTimer;

        Plasma::Animation *m_fadeIn;
        Plasma::Animation *m_fadeOut;

        Plasma::Applet *m_parent;

        static QHash<quint64, ActionsBox*> *_instances;
};

#endif // I_ACTIONSBOX_H
