/***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "EngineAboutDialog.h"

#include <QDebug>

EngineAboutDialog::EngineAboutDialog(const KService::Ptr & dataEngine, QWidget *parent, Qt::WFlags flags)
    :KDialog(parent, flags)
{
    QWidget *mw = new QWidget();
    mainWidget.setupUi(mw);

    const KService *eng = dataEngine.data();

    mainWidget.ktitlewidget->setPixmap(eng->icon(), KTitleWidget::ImageLeft);
    mainWidget.ktitlewidget->setText(eng->name());

    // Tab1
    QString comment("<b>Version %1</b><br />Using %2");

    mainWidget.ktitlewidget->setComment(comment.arg(eng->property("X-KDE-PluginInfo-Version").toString()).arg(QString(KDE_VERSION_STRING)));

    QString aboutPageText = eng->comment() + "<br />";

    QString homePage = eng->property("X-KDE-PluginInfo-Website").toString();

    if( !homePage.isEmpty() )
    {
        aboutPageText += "<br />" + QString("<a href=\"%1\">%1</a>").arg(homePage) + "<br />";
    }

    mainWidget.label->setWordWrap(true);
    mainWidget.label->setOpenExternalLinks(true);
    mainWidget.label->setTextInteractionFlags(Qt::TextBrowserInteraction);
    mainWidget.label->setText(aboutPageText);

    // Tab2
    QString authorPageText;
    QString mail = eng->property("X-KDE-PluginInfo-Email").toString();

    if(!mail.isEmpty())
    {
        authorPageText += i18n("Please report bugs to: ") + "<br />" + QString("<a href=\"%1\">%1</a>").arg(mail) + "<br />";
    }

    QString author = eng->property("X-KDE-PluginInfo-Author").toString();

    if( !author.isEmpty() )
    {
        authorPageText += "<br />" + author + "<br />";

        if( !mail.isEmpty() )
        {
            authorPageText += "   " + QString("<a href=\"%1\">%1</a>").arg(mail);
        }
    }


    mainWidget.label_2->setWordWrap(true);
    mainWidget.label_2->setOpenExternalLinks(true);
    mainWidget.label_2->setTextInteractionFlags(Qt::TextBrowserInteraction);
    mainWidget.label_2->setText(authorPageText);

    setMainWidget(mw);

    setButtons(KDialog::Ok);
    setDefaultButton(KDialog::Ok);
}

EngineAboutDialog::~EngineAboutDialog()
{
}
