/***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef I_ERRORBOX_H
#define I_ERRORBOX_H

#include <QTimer>
#include <QHash>

#include <Plasma/Frame>
#include <Plasma/Animation>
#include <Plasma/Label>
#include <Plasma/Applet>

class ErrorBox: public Plasma::Frame
{
    Q_OBJECT

    enum State {
        Initialized = 0,
        FadingIn = 1,
        Running = 2,
        FadingOut = 3,
        Dead = 4,
        Count = 5
    };

    public:
        static ErrorBox* Instance(Plasma::Applet *parent);

        void exec(const QString &errMsg);
        void kill();

    protected:
        explicit ErrorBox(Plasma::Applet *parent = 0);
        virtual ~ErrorBox();


    private slots:
        void fadeInFinished();
        void fadeOutFinished();
        void lifeTimeout();

    private:
        State m_state;

        QTimer m_lifetime;

        Plasma::Label *m_errorLabel;
        Plasma::Animation *m_fadeIn;
        Plasma::Animation *m_fadeOut;

        Plasma::Applet *m_parent;

        static QHash<quint64, ErrorBox*> *_instances;
};

#endif // I_ERRORBOX_H
