/***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "configdialog.h"
#include "EngineAboutDialog.h"

#include <QDebug>

// Plasma
#include <Plasma/Applet>
#include <Plasma/DataEngine>

//KDE
#include <KServiceTypeTrader>
#include <KAboutData>
#include <KAboutApplicationDialog>

#define cMsInMin            ((uint)60000)


ConfigDialog::ConfigDialog(QWidget *parent):QObject(parent)
{
    appSettings = new QWidget();
    appUi.setupUi(appSettings);

    appUi.frameWidth->setSuffix(ki18n("px"));
    appUi.frameOpacity->setSuffix(ki18n("%"));

    appUi.shadowSize->setSuffix(ki18n("px"));
    appUi.shadowOpacity->setSuffix(ki18n("%"));

    appUi.roundness->setSuffix(ki18n("px"));

    potdSettings = new QWidget();
    potdUi.setupUi(potdSettings);

    potdUi.infoButton->setIcon(KIcon("dialog-information"));
    connect(potdUi.infoButton, SIGNAL(clicked()), this, SLOT(showInfoDialog()));

    actionsBox = new QWidget();
    m_actionsBoxUi.setupUi( actionsBox );
}

//======================================================================================================================
// General dialog
//======================================================================================================================
void ConfigDialog::setDataEngines(const QStringList & dataEngines, const QString &selected)
{
    potdUi.dataEnginesCb->addItems(dataEngines);

    potdUi.dataEnginesCb->setCurrentItem(selected);
}

QString ConfigDialog::getDataEngine() const
{
    return potdUi.dataEnginesCb->currentText();
}

void ConfigDialog::setUpdateInterval(uint interval)
{
    interval /= cMsInMin; // convert to minutes

    QTime tm(interval / 60, interval % 60);

    potdUi.updateIntervalTE->setTime(tm);
}

uint ConfigDialog::getUpdateInterval() const
{
    uint interval;

    interval = potdUi.updateIntervalTE->time().hour() * 60 + potdUi.updateIntervalTE->time().minute();

    interval *= cMsInMin;

    return interval;
}

void ConfigDialog::setSlideShowEnabled( bool enabled )
{
    potdUi.slideShowGB->setChecked( enabled );
}

bool ConfigDialog::getSlideShowEnabled() const
{
    return potdUi.slideShowGB->isChecked();
}

void ConfigDialog::setSlideInterval( uint interval )
{
    potdUi.slideShowInterval->setValue( interval / cMsInMin );
}

uint ConfigDialog::getSlideInterval() const
{
    return potdUi.slideShowInterval->value() * cMsInMin;
}

//======================================================================================================================
// Appearance dialog
//======================================================================================================================
bool ConfigDialog::getHasFrame() const
{
    return appUi.frameGroupBox->isChecked();
}

void ConfigDialog::setHasFrame(bool checked)
{
    appUi.frameGroupBox->setChecked(checked);
}

int ConfigDialog::getFrameWidth() const
{
    return appUi.frameWidth->value();
}

void ConfigDialog::setFrameWidth(int width)
{
    if((width > 0) && (width <= 50))
    {
        appUi.frameWidth->setValue(width);
    }
}

int ConfigDialog::getFrameOpacity() const
{
    return appUi.frameOpacity->value();
}

void ConfigDialog::setFrameOpacity(int op)
{
    if((op >= 0) && (op <= 100))
    {
        appUi.frameOpacity->setValue(op);
    }
}

QColor ConfigDialog::getFrameColor() const
{
    return appUi.frameColor->color();
}

void ConfigDialog::setFrameColor(QColor &c)
{
    appUi.frameColor->setColor(c);
}

bool ConfigDialog::getHasShadow() const
{
    return appUi.shadowGroupBox->isChecked();
}

void ConfigDialog::setHasShadow(bool checked)
{
    appUi.shadowGroupBox->setChecked(checked);
}

int ConfigDialog::getShadowSize() const
{
    return appUi.shadowSize->value();
}

void ConfigDialog::setShadowSize(int size)
{
    if((size > 0) && (size <= 50))
    {
        appUi.shadowSize->setValue(size);
    }
}

int ConfigDialog::getShadowOpacity() const
{
    return appUi.shadowOpacity->value();
}

void ConfigDialog::setShadowOpacity(int op)
{
    if((op >= 0) && (op <= 100))
    {
        appUi.shadowOpacity->setValue(op);
    }
}

QColor ConfigDialog::getShadowColor() const
{
    return appUi.shadowColor->color();
}

void ConfigDialog::setShadowColor(QColor &c)
{
    appUi.shadowColor->setColor(c);
}

bool ConfigDialog::getRoundedCorners() const
{
    return appUi.rcGroupBox->isChecked();
}

void ConfigDialog::setRoundedCorners(bool checked)
{
    appUi.rcGroupBox->setChecked(checked);
}

int ConfigDialog::getRoundingFactor() const
{
    return appUi.roundness->value();
}

void ConfigDialog::setRoundingFactor(int rd)
{
    if((rd >= 0) && (rd <= 100))
    {
        appUi.roundness->setValue(rd);
    }
}

bool ConfigDialog::getSmoothScaling() const
{
    return appUi.smoothScaling->isChecked();
}

void ConfigDialog::setSmoothScaling(bool ss)
{
    appUi.smoothScaling->setChecked(ss);
}

bool ConfigDialog::getAnamatedTransition() const
{
    return potdUi.animatedTransition->isChecked();
}

void ConfigDialog::setAnamatedTransition(bool bAnimated)
{
    potdUi.animatedTransition->setChecked(bAnimated);
}

bool ConfigDialog::getShowActions() const
{
    return m_actionsBoxUi.showActionBox->isChecked();
}

void ConfigDialog::setShowActions( bool show  )
{
     m_actionsBoxUi.showActionBox->setChecked( show );
}

int ConfigDialog::getActionBoxPosition() const
{
    if ( true == m_actionsBoxUi.positionTopLeft->isChecked() )
    {
        return 0;
    }

    if ( true == m_actionsBoxUi.positionTopCenter->isChecked() )
    {
        return 1;
    }

    if ( true == m_actionsBoxUi.positionTopRight->isChecked() )
    {
        return 2;
    }

    if ( true == m_actionsBoxUi.positionBottomLeft->isChecked() )
    {
        return 3;
    }

    if ( true == m_actionsBoxUi.positionBottomCenter->isChecked() )
    {
        return 4;
    }

    if ( true == m_actionsBoxUi.positionBottomRight->isChecked() )
    {
        return 5;
    }

    return 5;
}

void ConfigDialog::setActionBoxPosition( int pos )
{
    switch ( pos )
    {
        case 0:
        {
            m_actionsBoxUi.positionTopLeft->setChecked( true );
            break;
        }

        case 1:
        {
            m_actionsBoxUi.positionTopCenter->setChecked( true );
            break;
        }

        case 2:
        {
            m_actionsBoxUi.positionTopRight->setChecked( true );
            break;
        }

        case 3:
        {
            m_actionsBoxUi.positionBottomLeft->setChecked( true );
            break;
        }

        case 4:
        {
            m_actionsBoxUi.positionBottomCenter->setChecked( true );
            break;
        }

        case 5:
        {
            m_actionsBoxUi.positionBottomRight->setChecked( true );
            break;
        }

        default:
        {
            Q_ASSERT_X( false, "ConfigDialog::setActionBoxPosition()", "Bad position" );
        }
    }
}

void ConfigDialog::showInfoDialog()
{
    // query for all Photo of the day data engines
    KService::List offers = KServiceTypeTrader::self()->query("Plasma/DataEngine",
                                                                  QString("Name == '%1' && [X-KDE-PluginInfo-Category] == 'PotdProvider' ").arg(getDataEngine()));

    //qDebug() << "Offers are " << offers.count();

    if( (!offers.isEmpty()) && (offers.count() == 1) )
    {
        KService::Ptr & dePtr = offers.first();

        EngineAboutDialog *ead = new EngineAboutDialog(dePtr, potdUi.infoButton);
        ead->exec();
        ead->deleteLater();
    }
    else
    {
        Q_ASSERT_X(false, "ConfigDialog::showInfoDialog()", "No data engine found.");
    }
}
