/***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef I_CONFIGDIALOG_H
#define I_CONFIGDIALOG_H

#include "ui_appearanceSettings.h"
#include "ui_PotD.h"
#include "ui_actionBoxConf.h"

class ConfigDialog: public QObject
{
    Q_OBJECT;

    public:
        ConfigDialog(QWidget *parent = 0);

        // general dialog
        void setDataEngines(const QStringList & dataEngines, const QString &selected);
        QString getDataEngine() const;

        void setUpdateInterval(uint interval);
        uint getUpdateInterval() const;

        void setSlideShowEnabled( bool enabled );
        bool getSlideShowEnabled() const;

        void setSlideInterval( uint interval );
        uint getSlideInterval() const;

        // appearance dialog
        Q_PROPERTY(bool hasFrame READ getHasFrame WRITE setHasFrame)
        Q_PROPERTY(int frameWidth READ getFrameWidth WRITE setFrameWidth)
        Q_PROPERTY(int frameOpacity READ getFrameOpacity WRITE setFrameOpacity)
        Q_PROPERTY(QColor frameColor READ getFrameColor WRITE setFrameColor)
        Q_PROPERTY(bool hasShadow READ getHasShadow WRITE setHasShadow)
        Q_PROPERTY(int shadowSize READ getShadowSize WRITE setShadowSize)
        Q_PROPERTY(int shadowOpacity READ getShadowOpacity WRITE setShadowOpacity)
        Q_PROPERTY(QColor shadowColor READ getShadowColor WRITE setShadowColor)
        Q_PROPERTY(bool roundedCorners READ getRoundedCorners WRITE setRoundedCorners)
        Q_PROPERTY(int roundness READ getRoundingFactor WRITE setRoundingFactor)
        Q_PROPERTY(bool smoothScaling READ getSmoothScaling WRITE setSmoothScaling)
        Q_PROPERTY(bool showActions READ getShowActions WRITE setShowActions)
        Q_PROPERTY(int actionBoxPosition READ getActionBoxPosition WRITE setActionBoxPosition)

        bool getHasFrame() const;
        void setHasFrame(bool checked);

        int getFrameWidth() const;
        void setFrameWidth(int width);

        int getFrameOpacity() const;
        void setFrameOpacity(int op);

        QColor getFrameColor() const;
        void setFrameColor(QColor &c);

        bool getHasShadow() const;
        void setHasShadow(bool checked);

        int getShadowSize() const;
        void setShadowSize(int size);

        int getShadowOpacity() const;
        void setShadowOpacity(int op);

        QColor getShadowColor() const;
        void setShadowColor(QColor &c);

        bool getRoundedCorners() const;
        void setRoundedCorners(bool checked);

        int getRoundingFactor() const;
        void setRoundingFactor(int rd);

        bool getSmoothScaling() const;
        void setSmoothScaling(bool ss);

        bool getAnamatedTransition() const;
        void setAnamatedTransition(bool bAnimated);

        bool getShowActions() const;
        void setShowActions( bool show  );
        
        int getActionBoxPosition() const;
        void setActionBoxPosition( int pos );

        QWidget *appSettings;
        QWidget *potdSettings;
        QWidget *actionsBox;

    private slots:
        void showInfoDialog();

    private:
        Ui::appearanceSettings appUi;
        Ui::uiPotD potdUi;
        Ui::actionBoxConf m_actionsBoxUi;
};

#endif // I_CONFIGDIALOG_H
