/***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef I_POTD_H
#define I_POTD_H

#include "ActionsBox.h"

// Qt

// Plasma
#include <Plasma/Applet>
#include <Plasma/DataEngine>

class QByteArray;
class QPixmap;
class QColor;
class QParallelAnimationGroup;
class QPropertyAnimation;

class KUrl;

class ConfigDialog;

class PhotoOfTheDay: public Plasma::Applet
{
    Q_OBJECT

    public:

        Q_PROPERTY(qreal transtionOpacity READ getTranstionOpacity WRITE setTranstionOpacity)

        PhotoOfTheDay(QObject *parent, const QVariantList &args);
        ~PhotoOfTheDay();

        void paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option, const QRect& contentsRect);

        void init();

        QSizeF contentSizeHint() const;
        QSizeF transitionSizeHint( const QPixmap &pm ) const;

        qreal getTranstionOpacity() const;
        void setTranstionOpacity(qreal opacity);

        int calcOffset();

    private:

        void updateToolTip();

        bool connectEngine(const QString & pluginName);
        void disconnectEngine();

        void loadDefaultImage();
        void redrawApplet(const QSizeF &newSize);

        void startTransitionAnimation();
        void stopTranstionAnimation();

        void startSlideShow();

    public slots:

        void createConfigurationInterface(KConfigDialog *parent);
        void dataUpdated(const QString& sourceName, const Plasma::DataEngine::Data& data);
        void setErrorMessage(const QString &errP);
        void animationFinished();
        void useNextEngine();
        void usePrevEngine();

    private slots:

        void hoverEnterEvent( QGraphicsSceneHoverEvent *event );
        void hoverLeaveEvent( QGraphicsSceneHoverEvent *event );


    protected slots:

        void configAccepted();


    protected:

        void constraintsEvent(Plasma::Constraints constraints);

    private:

        bool m_bAnimating;

        // configurations

        // General tab
        bool m_animatedTransition;
        uint m_updateInterval;

        // Appearance tab
        bool m_hasFrame;
        bool m_hasShadow;
        bool m_hasRoundCorners;
        bool m_smoothScaling;

        int m_frameWidth;
        int m_frameWidthConfig;
        int m_shadowSize;
        int m_shadowSizeConfig;
        int m_roundingFactor;

        int m_frameOpacity;
        int m_shadowOpacity;

        QColor m_frameColor;
        QColor m_shadowColor;

        // Actions box tab
        bool m_bShowActions;
        ActionsBox::ActionBoxPosition m_actionsBoxPosition;

        // Slide show
        bool m_slideShowEnabled;
        uint m_slideInterval;
        QTimer *m_slideTimer;

        // data engine
        QStringList m_engineSources;
        Plasma::DataEngine *m_pEngine;


        // data from data engine
        QString m_title; // photo title
        QString m_info; // photo description
        QPixmap *m_pPhoto; // photo
        QPixmap *m_pOldPhoto; // old photo. saved only during transtions

        // drawn photo (for optimization when need repaint)
        QPixmap m_drawnApplet;

        // internal
        qreal m_transtionOpacity;

        // transition animation
        QParallelAnimationGroup *m_pAnimGroup;
        QPropertyAnimation *m_pSizeAnim;
        QPropertyAnimation *m_pOpacityAnim;

        ConfigDialog *m_configDialog;
};

#endif // I_POTD_H
