/***************************************************************************
 *   Copyright  2009 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef I_ABSTRACTPROVIDER_H
#define I_ABSTRACTPROVIDER_H

// Qt
#include <QString>
#include <QPixmap>

// KDE
#include <KUrl>

// Plasma
#include <Plasma/DataEngine>

// Solid
#include <Solid/Networking>

// shared constants
#include "../potd_shared.h"

class AbstractProvider: public Plasma::DataEngine
{
    Q_OBJECT

    public:
        AbstractProvider(QObject *parent, const QVariantList &args);
        ~AbstractProvider() {}

    protected:

        virtual bool sourceRequestEvent ( const QString &source ) { Q_UNUSED( source ); return false; }
        virtual bool updateSourceEvent ( const QString &source ) { Q_UNUSED( source) ; return false; }

        void setPhotoSourceData();
        void setError( const QString & err );
        void restore();

        void saveCache() const;
        void saveText() const;
        void savePixmap() const;

        bool loadCache();
        void loadText();
        bool loadPixmap();


    private slots:
        void networkStatusChangedSlot(Solid::Networking::Status status);

    private:

    protected:
        // provided data
        QString m_title; // photo title
        QString m_info; // proho description
        QPixmap m_photo; // photo
        KUrl m_associatedUrl; // associated application url

        KUrl m_OldUrl; // url of previous photo
};

#endif // I_ABSTRACTPROVIDER_H
