 /***************************************************************************
 *   Copyright  2010 by Yann Nave <yannux@onbebop.net>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

 // KDE
 #include <KIO/Job>
 
 #include <BonjourMadame.h>
 
const char *cMainPageUrl = "http://www.bonjourmadame.fr/";
const char *cPhotosWebsite = "http://www.bonjourmadame.fr/photo/";
const char *cPhotoTag = "<a href=\"http://www.bonjourmadame.fr/photo/";


BonjourMadame::BonjourMadame(QObject *parent, const QVariantList &args):AbstractProvider(parent, args)
{
  m_associatedUrl = KUrl( cMainPageUrl );
}

BonjourMadame::~BonjourMadame()
{
}

// called on first source request
bool BonjourMadame::sourceRequestEvent (const QString &source)
{
    bool result = true;

    if(I18N_NOOP(cPotdPhotoSource) == source)
    {
        // set empty data for inialization
        setData(cPotdPhotoSource, DataEngine::Data());

        // and update data
        return updateSourceEvent(cPotdPhotoSource);
    }
    else if(I18N_NOOP(cPotdErrorSource) == source)
    {
        setData(cPotdErrorSource, DataEngine::Data());
    }
    else
    {
        Q_ASSERT_X(false, "BonjourMadame::sourceRequestEvent()", "Invalid source!");

        return result = false;
    }


    return result;
}

bool BonjourMadame::updateSourceEvent (const QString &source)
{
    bool result = true;

    if( I18N_NOOP(cPotdPhotoSource) == source )
    {
        KIO::StoredTransferJob *job = KIO::storedGet(KUrl(cMainPageUrl), KIO::Reload, KIO::HideProgressInfo);

        connect(job, SIGNAL(result(KJob*)), this, SLOT(mainPageLoaded(KJob*)));
    }
    else if(I18N_NOOP(cPotdErrorSource) == source)
    {
        // do nothing for now
    }
    else
    {
        //Q_ASSERT_X(false, "BonjourMadame::updateSourceEvent", "Unexpected source");
        result = false;
    }

    return result;
}

void BonjourMadame::mainPageLoaded( KJob *job )
{
    if( job->error() )
    {
        setError( "[FATAL ERROR]BonjourMadame Engine: Unable to connect.\nLoading from cache ..." );
        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);
        parseMainPage( storedJob->data() );
    }
}

void BonjourMadame::parseMainPage(QByteArray source)
{
    int photoTagStart = source.indexOf("<div class=\"photo\">") + (int)strlen("<div class=\"photo\">");
    int imgStart = source.indexOf("<img src=\"", photoTagStart) + (int)strlen("<img src=\"");
    int imgEnd =  source.indexOf("\"", imgStart);
    
    QString imgSrc = source.mid(imgStart, imgEnd - imgStart);
    
    KUrl newUrl = KUrl( imgSrc );

    if(!newUrl.isValid())
    {
        setError( QString("[ERROR]Invalid url: %1.").arg( newUrl.url() ) );
        // TODO: add 'loading from cache ...' only if m_photo.isNull

        restore();

        return;
    }

    if(newUrl != m_OldUrl)
    {
        // we have new photo

        m_OldUrl = newUrl;

        // get photographer
        int photographerStart = source.indexOf("<div class=\"caption\">") + (int)strlen("<div class=\"caption\">");
        int photographerEnd = source.indexOf("</div>", photographerStart);

        QString photographer = QString::fromUtf8(source.mid(photographerStart, photographerEnd - photographerStart).trimmed());
	
        photographer.replace(QRegExp("<[^>]*>"), "");
        m_info = photographer;

        m_title = QString::fromUtf8("Bonjour Madame ! ");

        KIO::StoredTransferJob *job = KIO::storedGet(m_OldUrl, KIO::Reload, KIO::HideProgressInfo);
        connect(job, SIGNAL(result(KJob*)), this, SLOT(photoDownloaded(KJob*)));
    }
    else
    {
        // photo is the same as old one
        setPhotoSourceData();
        forceImmediateUpdateOfAllVisualizations();
    }
}


void BonjourMadame::photoDownloaded(KJob *job)
{
    if( job->error() )
    {
        setError( QString("[FATAL ERROR]Unable to download pohto: %1.\nLoading from cache ...").arg( job->errorText() ) );

        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);


        if( m_photo.loadFromData(storedJob->data()) )
        {
            // everything is ok
            setPhotoSourceData();
            scheduleSourcesUpdated();
            saveCache();
        }
        else
        {
            setError( "[ERROR]Unable to load photo. Loading from cache ..." );
            restore();
        }
    }
}

K_EXPORT_PLASMA_DATAENGINE(BonjourMadame, BonjourMadame)

#include "BonjourMadame.moc"
