/***************************************************************************
*   Copyright 2009-2010 by Nikolay Nikolov <nikolay.fatal.error@gmail.com>*
*             2009      by The Wolf                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
***************************************************************************/

#include "Wiki.h"

// Qt
#include <QHash>

// KDE
#include <KIO/Job>
#include <KGlobal>

static const char *cPhotosWebsite = "http://upload.wikimedia.org/wikipedia/commons/thumb/";

Wiki::Wiki(QObject *parent, const QVariantList &args):AbstractProvider(parent, args)
{
   //all the wikipedia languages that somehow match with kde locales availiable
    QHash<QString, QUrl> WikiPagesMapping = QHash<QString, QUrl>();

//    WikiPagesMapping["ar"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:صورة_اليوم" );
//    WikiPagesMapping["ast"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Imaxen_del_día" );
//    WikiPagesMapping["be"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Выява_дня" );
//    WikiPagesMapping["bg"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Картинка_на_деня" );
    WikiPagesMapping["hr"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Slika_dana" );
    WikiPagesMapping["ca"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Imatge_del_dia" );
//    WikiPagesMapping["cs"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Obrázek_dne" );
    WikiPagesMapping["da"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Dagens_billede" );
    WikiPagesMapping["de"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Bild_des_Tages" );
//    WikiPagesMapping["el"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Εικόνα_της_ημέρας" );
    WikiPagesMapping["en"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Picture_of_the_day" );
    WikiPagesMapping["eo"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Bildo_de_la_tago" );
//    WikiPagesMapping["es"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:La_imagen_del_día" );
//    WikiPagesMapping["et"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:P%C3%A4eva_pilt" );
    WikiPagesMapping["eu"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Eguneko_irudia" );
    WikiPagesMapping["fr"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Image_du_jour" );
//    WikiPagesMapping["ga"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Pictiúr_an_lae" );
//    WikiPagesMapping["gl"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Imaxe_do_día" );
//    WikiPagesMapping["gu"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:આજનું_ચિત્ર" );
//    WikiPagesMapping["he"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:תמונת_היום" );
    WikiPagesMapping["hr"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Slika_dana" );
//    WikiPagesMapping["hu"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:A_nap_képe" );
    WikiPagesMapping["it"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Immagine_del_giorno" );
//    WikiPagesMapping["ja"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:今日の一枚" );
//    WikiPagesMapping["kk"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Тәулік_суреті" );
//    WikiPagesMapping["ko"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:오늘의_그림" );
//    WikiPagesMapping["lt"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Dienos_rinktinis_paveikslėlis" );
//    WikiPagesMapping["mk"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Слика_на_денот" );
    WikiPagesMapping["nb"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Dagens_bilde" );
    WikiPagesMapping["nl"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Afbeelding_van_de_dag" );
    WikiPagesMapping["nn"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Dagens_bilete" );
//    WikiPagesMapping["pl"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Grafika_na_dziś" );
    WikiPagesMapping["pt"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Imagem_do_dia" );
    WikiPagesMapping["ro"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Poza_zilei" );
//    WikiPagesMapping["ru"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Изображение_дня" );
//    WikiPagesMapping["sk"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Obrázok_dňa" );
    WikiPagesMapping["sl"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Slika_dneva" );
//    WikiPagesMapping["sr"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Слика_дана" );
//    WikiPagesMapping["fi"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Päivän_kuva" );
    WikiPagesMapping["sv"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Dagens_bild" );
//    WikiPagesMapping["th"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:ภาพประจำวัน" );
//    WikiPagesMapping["tr"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Günün_resmi" );
//    WikiPagesMapping["uk"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:Зображення_дня" );
//    WikiPagesMapping["zh-hans"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:今日图片" );
//    WikiPagesMapping["zh-hant"] = QUrl( "http://commons.wikimedia.org/wiki/Commons:每日圖片" );

    // get language
    m_lang = KGlobal::locale()->language();

    if ( "zh_CN" == m_lang )
    {
        m_lang = "zh-hans";
    }
    else if ( "zh_TW" == m_lang )
    {
        m_lang = "zh-hant";
    }

    // get wiki page
    m_wikiPage = WikiPagesMapping.value(m_lang, QUrl());

    if ( m_wikiPage.isEmpty() )
    {
        // no mapping for current language. Load the default one (en)
        m_lang = "en";
        m_wikiPage = WikiPagesMapping.value( m_lang );
    }

    m_associatedUrl = m_wikiPage;

    //qDebug() << "Choosen language is " << m_lang << ", wiki page is " << m_wikiPage;
}

Wiki::~Wiki()
{
}


// called on first source request
bool Wiki::sourceRequestEvent (const QString &source)
{
    bool result = true;

    if(I18N_NOOP(cPotdPhotoSource) == source)
    {
        // set empty data for inialization
        setData(cPotdPhotoSource, DataEngine::Data());

        // and update data
        return updateSourceEvent(cPotdPhotoSource);
    }
    else if(I18N_NOOP(cPotdErrorSource) == source)
    {
        setData(cPotdErrorSource, DataEngine::Data());
    }
    else
    {
        Q_ASSERT_X(false, "Wiki::sourceRequestEvent()", "Invalid source!");

        return result = false;
    }


    return result;
}

bool Wiki::updateSourceEvent (const QString &source)
{
    bool result = true;

    if( I18N_NOOP(cPotdPhotoSource) == source )
    {
        KIO::StoredTransferJob *job = KIO::storedGet( KUrl( m_wikiPage ), KIO::Reload, KIO::HideProgressInfo );

        connect( job, SIGNAL( result( KJob* ) ), this, SLOT( mainPageLoaded( KJob* ) ) );

    }
    else if( I18N_NOOP(cPotdErrorSource) == source )
    {
        // do nothing for now
    }
    else
    {
        Q_ASSERT_X(false, "Wiki::updateSourceEvent", "Unexpected source");
        result = false;
    }

    return result;
}

void Wiki::mainPageLoaded( KJob *job )
{
    if(job->error())
    {
        setError( "[FATAL ERROR]Wiki Engine: Unable to connect.\nLoading from cache ..." );

        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);
        parseMainPage( storedJob->data() );
    }
}

void Wiki::parseMainPage( QByteArray source )
{
    int photoStart = source.indexOf("src=\"http://upload.wikimedia.org/wikipedia/commons/thumb/") + (int)strlen("src=\"http://upload.wikimedia.org/wikipedia/commons/thumb/");
    int photoEnd = source.indexOf("\"", photoStart);

    //make a change in the url to download the 600px version, don't know if it's useful
    //400px, 500px are also valid, maybe better?
    QString tmp = QString(cPhotosWebsite + source.mid(photoStart, photoEnd - photoStart));
    tmp.replace(QString("300px"), QString("600px"));
    KUrl newUrl = KUrl(tmp);


    if( !newUrl.isValid() )
    {
        setError( QString("[ERROR]Invalid url: %1.").arg( newUrl.url() ) );
        // TODO: add 'loading from cache ...' only if m_photo.isNull

        restore();

        return;
    }

    if(newUrl != m_OldUrl)
    {
        // we have new photo

        m_OldUrl = newUrl;

        //gets the name of the image, remove the extension and set it as title
        int titleStart = source.indexOf("<img alt=\"") + strlen("<img alt=\"");
        int titleEnd = source.indexOf("\"", titleStart);
        m_title = source.mid( titleStart, titleEnd - titleStart );
        m_title.remove( m_title.lastIndexOf("."), m_title.length() );

        //now it takes the description based on the system locale, cooler
        tmp = QString("class=\"description " + m_lang + "\" xml:lang=\"" + m_lang + "\">");

        int infoStart = source.indexOf(tmp.toUtf8()) + tmp.length();
        int infoEnd = source.indexOf("</span>", infoStart);
        m_info = source.mid(infoStart, infoEnd - infoStart);
        //remove an html tag
        m_info.replace("</a>","");

        // strip html tags
        m_info.replace(QRegExp("<[^>]*>"), "");

        KIO::StoredTransferJob *job = KIO::storedGet( m_OldUrl, KIO::Reload, KIO::HideProgressInfo );
        connect(job, SIGNAL(result(KJob*)), this, SLOT(photoDownloaded(KJob*)));
    }
    else
    {
        // photo is the same as old one
        setPhotoSourceData();
        forceImmediateUpdateOfAllVisualizations();
    }
}

void Wiki::photoDownloaded(KJob *job)
{
    if(job->error())
    {
        setError( QString("[FATAL ERROR]Unable to download pohto: %1.\nLoading from cache ...").arg( job->errorText() ) );

        restore();
    }
    else
    {
        KIO::StoredTransferJob *storedJob = qobject_cast<KIO::StoredTransferJob*>(job);

        if( m_photo.loadFromData(storedJob->data()) )
        {
            // everything is ok
            setPhotoSourceData();
            scheduleSourcesUpdated();
            saveCache();
        }
        else
        {
            setError( "[ERROR]Unable to load photo. Loading from cache ..." );
            restore();
        }
    }
}

K_EXPORT_PLASMA_DATAENGINE(WikiPotd, Wiki)

#include "Wiki.moc"

